import flask
import paho.mqtt.client as mqtt
import logging
import json
from collections import defaultdict

logging.basicConfig(level=logging.DEBUG, format='[%(asctime)-15s] %(name)-10s %(levelname)7s: %(message)s')


class SpejsiotDevice(object):
    node_id = None
    properties = None
    endpoints = None

    def __init__(self, node_id):
        self.node_id = node_id
        self.properties = {}
        self.endpoints = defaultdict(dict)

    def dictify(self):
        return dict(self.properties, **self.endpoints)


class SpejsiotManager(mqtt.Client):
    devices = dict()
    logger = logging.getLogger('manager')

    def __init__(self):
        super(SpejsiotManager, self).__init__()

    def run(self, broker, port):
        self.connect(broker, port, 60)
        self.loop_start()

    def on_connect(self, client, userdata, flags, rc):
        self.logger.info('Connected, rc: %d', rc)
        self.subscribe('iot/+/+')
        self.subscribe('iot/+/+/+')

    def on_message(self, client, userdata, msg):
        try:
            self.logger.info('Message received %s: %s', msg.topic, msg.payload)
            _, node_id, topic = msg.topic.split('/', 2)

            if node_id not in self.devices.keys():
                self.devices[node_id] = SpejsiotDevice(node_id)

            if topic.startswith('$'):
                self.devices[node_id].properties[topic] = msg.payload
            else:
                endpoint, prop = topic.split('/', 1)
                self.devices[node_id].endpoints[endpoint][prop] = msg.payload

            print(topic)
        except:
            self.logger.exception('fuckup')

    def handle_request(self, node_id, endpoint, prop, value):
        self.publish('iot/%s/%s/%s/set' % (node_id, endpoint, prop), value, retain=True)

        return True

manager = SpejsiotManager()
app = flask.Flask(__name__)
app.config['TEMPLATES_AUTO_RELOAD'] = True


class EndpointRenderer(object):
    @classmethod
    def render(cls, device, endpoint):
        return ''


class DefaultRenderer(EndpointRenderer):
    @classmethod
    def render(cls, device, endpoint):
        return flask.Markup(flask.render_template(
            'endpoints/%s.html' % endpoint.get('$type'),
            endpoint=endpoint, device=device))


endpoint_renderers = {
    None: DefaultRenderer
    }


@app.context_processor
def utility_processor():
    def render_endpoint(device, endpoint):
        t = endpoint.get('$type', None)
        if t in endpoint_renderers:
            return endpoint_renderers[t].render(device, endpoint)
        else:
            return endpoint_renderers[None].render(device, endpoint)
    return {
        'render_endpoint': render_endpoint,
        }

@app.route('/')
def index():
    return flask.render_template('index.html', devices=manager.devices)

@app.route('/api/1/devices')
def api_devices():
    return flask.jsonify({k: v.dictify() for k, v in manager.devices.items()})

@app.route('/api/1/device/<node_id>/<endpoint>/<prop>/<value>')
def device_write(node_id, endpoint, prop, value):
    return flask.jsonify(manager.handle_request(node_id, endpoint, prop, value))


if __name__ == "__main__":
    manager.run('127.0.0.1', 1883)
    app.run()
